
--Copyright (c) 1993 Ohio Board of Regents and the University of
--Cincinnati.  All Rights Reserved.


--example in the construction of petri nets using the vhdl design
--entities and package of in this directory.  this example
--instantiates the following petri net.


--    P1(3)      P2(5)
--    (place)    (place)
--      |          |
--      |          +-------+--+
--      |          |       |  |
--      +-------------+    |  |
--      |          |  |    |  |
--      |          |  |    |  |
--      |  +------ +  |    |  |
--      |  |       |  |    |  |
--      |  |       |  |    |  |
--      |  |       |  |    |  |
--      |  |       |  |    |  |
--     ------     ------   |  |
--       T1         T2     |  |
--     (trans)    (trans)  |  |
--        |          |     |  |
--        |          |     |  |
--        |          |     |  |
--        |  +-------+     |  |
--        |  |             |  |
--        |  |             |  |
--        |  |             |  |
--        P3(0)            |  |
--        (place)          |  |
--          |              |  |
--          |              |  |
--          +-----------+  |  |
--                      |  |  |
--                      |  |  |
--                      |  |  |
--                      |  |  |
--                     ---------
--                       T3
--                     (trans)
--                        |
--                        |
--                        |
--                       P4(0)
--                       (place)


library petri;
use petri.petri.all;
entity petri_use is

end;


architecture demo of petri_use is

signal arc1, arc2, arc3, arc4, arc5, 
       arc6, arc7, arc8, arc9, arc10,
       arc11, arc12, arc13, arc14, arc15 : arc bus; 

component place_2_2

    generic (initial_marking : integer;
             place_name      : string;
             debug_cntrl     : debug := trace_tokens);

    port (in_arc1, in_arc2, out_arc1, out_arc2 : inout arc bus );

end component;


component place_3_4

    generic (initial_marking : integer;
             place_name      : string;
             debug_cntrl     : debug := trace_tokens);

    port (in_arc1, in_arc2, in_arc3, out_arc1, out_arc2, out_arc3,
                           out_arc4 : inout arc bus);
    
end component;

component transition_1_3

    generic (transition_name : string;
             debug_cntrl     : debug := firing);

    port (in_arc1                      : inout arc bus;
          out_arc1, out_arc2, out_arc3 : inout arc bus);

end component;

component transition_2_1

    generic (transition_name : string;
             debug_cntrl     : debug := firing);

    port (in_arc1,  in_arc2  : inout arc bus;
          out_arc1           : inout arc bus);

end component;

component transition_2_2

    generic (transition_name : string;
             debug_cntrl     : debug := firing);

    port (in_arc1,  in_arc2  : inout arc bus;
          out_arc1, out_arc2 : inout arc bus);

end component;

component transition_3_1

    generic (transition_name : string;
             debug_cntrl     : debug := firing);

    port (in_arc1,  in_arc2,  in_arc3  : inout arc bus;
          out_arc1                     : inout arc bus);

end component;


for P1 : place_2_2 use entity work.place_2_2(structure);
for P2 : place_2_2 use entity work.place_2_2(structure);
for P3 : place_3_4 use entity work.place_3_4(structure);

for T1     : transition_1_3 use entity work.transition_1_3(structure);
for T2     : transition_2_2 use entity work.transition_2_2(structure);
for T3     : transition_3_1 use entity work.transition_3_1(structure);
for T4     : transition_2_1 use entity work.transition_2_1(structure);

begin

P1 : place_2_2 
     generic map (1, "P1")
     port map (arc1, arc2, arc3, arc4);
P2 : place_2_2
     generic map (0, "P2")
     port map (arc5, arc6, arc7, arc8);
T1 : transition_1_3
     generic map ("T1")
     port map (arc3, arc2, arc10, arc11);
T2 : transition_2_2
     generic map ("T2")
     port map (arc4, arc15, arc5, arc9);
P3 : place_3_4
     generic map (0, "P3")
     port map (arc9, arc10, arc11, arc12, arc13, arc14, arc15);
T3 : transition_3_1
     generic map ("T3")
     port map (arc7, arc12, arc13, arc6);
T4 : transition_2_1
     generic map ("T4")
     port map (arc8, arc14, arc1);

end;
