--#########################################################################
--
--    perfsim2 - performance simulator
--
--    author:   Peter Frey
--    date:     Aug. 93
--    version:  1.0
--
--    Acknowledgements:  This program is a rewrite of the PERFSIM program
--                       written by Guru Prasad
--
--    (c) Copyright University of Cincinnati, 1993    All Rights Reserved
--
--#########################################################################
use STD.TEXTIO.all;
package math is

  function LOG(x: in REAL) return REAL;
  function intpow(x: in real; y: in integer) return real;
  function POWER(x: in REAL; y: in REAL) return REAL;
  function FACTORIAL(x: in REAL) return REAL;
  function sqrt(x: in real) return real;
  procedure ran(dist : in integer;par1 : in integer;from : in real;
		upto : in real;prev : inout integer;idum : out real);
  function len(first : integer;last : integer; length : integer)
                                        return integer;
  procedure normdist(mu : in integer; si : in integer; 
                     from : in real; upto : in real;
  	             prev : inout integer; idum : out real);
  procedure chi2dist(v: in integer; 
                     from: in real; upto : in real;
  	  	     prev: inout integer; idum : out real);

end math;

package body math is

        function len(first : integer;last : integer; length : integer)
                                        return integer is

        variable count : integer := 0;
        variable tmp : integer := 0;

        begin

                count := last;
                while (count /= first) loop
                        tmp := tmp + 1;
                        count := (count+1) MOD length;
                        if(count = 0) then
                                count := 1;
                        end if;
                end loop;
                return tmp;

        end len;


        procedure ran(dist : in integer;par1 : in integer;from : in real;
		      upto : in real;prev : inout integer;idum : out real) is

        variable im : integer := 259200;  -- parameter of the LCG
        variable ia : integer := 7141;    -- are taken out of "Numerical
        variable ic : integer := 54773;   -- recipes in C"
        variable temp : real;
	variable help : real;

        begin

		if(dist = 1) then
	                prev := (prev*ia+ic) mod im; 
			idum := (from+((upto-from)*real(prev))/real(im));
		elsif(dist = 2) then
			help:=from-1.0;
			WHILE((help<from) OR (help>upto)) LOOP
		                prev := (prev*ia+ic) mod im; 
				temp := real(prev)/real(im);
				help := (-real(par1)*LOG(temp))-0.5;
			END LOOP;
			idum:=help;
		end if;

        end ran;

        procedure normdist(mu: in integer; si: in integer; 
                           from: in real; upto : in real;
			   prev: inout integer; idum : out real) is
	variable im : integer := 259200; -- parameter of the LCG
	variable ia : integer := 7141;   -- are taken out of "Numerical 
	variable ic : integer := 54773;  -- recipes in C"
	variable temp: real;

	begin
	temp:=from-1.0;
	WHILE ((temp<from) OR (temp>upto)) LOOP
		prev := (prev*ia+ic) mod im;
		temp := real(prev)/real(im);                --U1
		prev := (prev*ia+ic) mod im;
		temp := temp + real(prev)/real(im);         --U2
		prev := (prev*ia+ic) mod im;
		temp := temp + real(prev)/real(im);         --U3
		prev := (prev*ia+ic) mod im;
		temp := temp + real(prev)/real(im);         --U4
		prev := (prev*ia+ic) mod im;
		temp := temp + real(prev)/real(im);         --U5
		prev := (prev*ia+ic) mod im;
		temp := temp + real(prev)/real(im);         --U6
		prev := (prev*ia+ic) mod im;
		temp := temp + real(prev)/real(im);         --U7
		prev := (prev*ia+ic) mod im;
		temp := temp + real(prev)/real(im);         --U8
		prev := (prev*ia+ic) mod im;
		temp := temp + real(prev)/real(im);         --U9
		prev := (prev*ia+ic) mod im;
		temp := temp + real(prev)/real(im);         --U10
		prev := (prev*ia+ic) mod im;
		temp := temp + real(prev)/real(im);         --U11
		prev := (prev*ia+ic) mod im;
		temp := temp + real(prev)/real(im);         --U12
		temp := (temp - 6.0 ) * real(si) + real(mu);
	END LOOP;
	idum:=temp;
	end normdist;

        procedure chi2dist(v: in integer; 
                           from: in real; upto : in real;
			   prev: inout integer; idum : out real) is
	variable im : integer := 259200; -- parameter of the LCG
	variable ia : integer := 7141;   -- are taken out of "Numerical 
	variable ic : integer := 54773;  -- recipes in C"
	variable temp: real;
	variable temp2: real := 0.0;
        variable count : integer := 0;

	begin
        temp2 := 0.0;
        count := 0;
	WHILE ( count < v ) LOOP
		prev := (prev*ia+ic) mod im;
		temp := real(prev)/real(im);                --U1
		prev := (prev*ia+ic) mod im;
		temp := temp + real(prev)/real(im);         --U2
		prev := (prev*ia+ic) mod im;
		temp := temp + real(prev)/real(im);         --U3
		prev := (prev*ia+ic) mod im;
		temp := temp + real(prev)/real(im);         --U4
		prev := (prev*ia+ic) mod im;
		temp := temp + real(prev)/real(im);         --U5
		prev := (prev*ia+ic) mod im;
		temp := temp + real(prev)/real(im);         --U6
		prev := (prev*ia+ic) mod im;
		temp := temp + real(prev)/real(im);         --U7
		prev := (prev*ia+ic) mod im;
		temp := temp + real(prev)/real(im);         --U8
		prev := (prev*ia+ic) mod im;
		temp := temp + real(prev)/real(im);         --U9
		prev := (prev*ia+ic) mod im;
		temp := temp + real(prev)/real(im);         --U10
		prev := (prev*ia+ic) mod im;
		temp := temp + real(prev)/real(im);         --U11
		prev := (prev*ia+ic) mod im;
		temp := (temp + real(prev)/real(im)) - 6.0; --U12
		temp2 := temp2 + (temp*temp);
		count := count + 1;
	END LOOP;
	idum:=temp2;
	end chi2dist;
	
  function LOG(x: in REAL) return REAL
  is
    variable j   : integer := 0;
    variable n, k: real := 0.0;
    variable a,y,z,l,z0,f0: real := 0.0;
    variable k1 : line;
  begin

    z := x;
    loop1:
    loop
      if z < 1.0 then
        z := z*10.0;
        n := n+1.0;
      elsif z >= 10.0 then
        z := z/10.0;
        k := k+1.0;
      end if;
      exit loop1 when z < 10.0 and z >= 1.0;
    end loop loop1;
    loop2:
    loop
      j := j + 1;
      l := l + 1.0;
      if z >=1.0 and z < 2.0 then
        z0 := 1.0;
        f0 := 0.0;
      elsif z >= 2.0 and z < 3.0 then
        z0 := 2.0;
        f0 := 0.69314718055994530941;
      elsif z >= 3.0 and z < 4.0 then
        z0 := 3.0;
        f0 := 1.09861228866810969139;
      elsif z >= 4.0 and z < 5.0 then
        z0 := 4.0;
        f0 := 1.38629436111989061883;
      elsif z >= 5.0 and z < 6.0 then
        z0 := 5.0;
        f0 := 1.60943791243410037460;
      elsif z >= 6.0 and z < 7.0 then
        z0 := 6.0;
        f0 := 1.79175946922805500081;
      elsif z >= 7.0 and z < 8.0 then
        z0 := 7.0;
        f0 := 1.94591014905531330510;
      elsif z >= 8.0 and z < 9.0 then
        z0 := 8.0;
        f0 := 2.07944154167983592825;
      elsif z >= 8.0 and z < 10.0 then
        z0 := 9.0;
        f0 := 2.19722457733621938279;
      end if;
        a := a + intpow((-1.0),(j-1))*intpow(((z/z0)-1.0),j)/real(l);
        y := intpow(((z/z0)-1.0),(j+1))*intpow((-1.0),j)/(l+1.0);
      exit loop2 when abs(y) < 1.0e-14;
    end loop loop2;
    a := a + f0;
    a := a*0.43429448190325182765;
    if n /= 0.0 then
      a := a - n ;
    elsif k /= 0.0 then
      a := a + k;
    end if;
  return a;
  end LOG;


  function intpow(x: in real; y: in integer) return real
  is
    variable z: real := 1.0;
    variable i: integer:=1;
  begin
    if (y=0) then
      return 1.0;
    end if;
    for i in 1 to y loop
      z := z * x;
    end loop;
    return z;
  end intpow;


  function POWER(x: in real; y: in real) return real
  is
file FILE2 :TEXT is out "STD_OUTPUT";
variable k : line;
    variable a,b,c,l: real := 1.0;
    variable j : real := 0.0;
    variable n : INTEGER := 0;
    variable m : integer := 1;
  begin
    b := 2.30258509299404568401*y*LOG(x);
    if abs(b) >=1.0 then
      loop0:
      loop
        b:=b/10.0;
        m:=m*10;
        exit loop0 when abs(b) < 1.0;
      end loop loop0;
    end if;
    if b > 0.0 then
      loop1:
      loop
        n := n+1;
        j := j+1.0;
        l := FACTORIAL(j);
        a := a + (b**n)/l;
        c := (b**(n+1))/(l*(j+1.0));
        exit loop1 when c < 1.0e-14;
      end loop loop1;
    elsif b < 0.0 then
      b := -b;
      loop2:
      loop
        n := n+1;
        j := j+1.0;
        a := a + (b**n)/FACTORIAL(j);
        c := (b**(n+1))/FACTORIAL(j+1.0);
        exit loop2 when c < 1.0e-14;
      end loop loop2;
      a := 1.0/a;
    elsif b = 0.0 then
      a:=1.0;
    end if;
    a:=a**m;
  return a;
  end POWER;


  function FACTORIAL(x: in real) return real is
    variable a,j: real := 1.0;
  begin
    loop1:
    loop
      a := a*j;
      exit loop1 when j = x;
      j := j + 1.0;
    end loop loop1;
  return a;
  end FACTORIAL;


  function sqrt(x: in real) return real is
    variable a: real;
    constant error: real:=1.0e-12;
file FILE2 : text is out "STD_OUTPUT";
variable k : line;

  begin
    a := 1.0;
    while abs(a-x/a) > error loop
      a := (a+x/a)/2.0;
    end loop;
    return a;
  end sqrt;

end math; 
