
--Copyright (c) 1993 Ohio Board of Regents and the University of
--Cincinnati.  All Rights Reserved.

--author: sidhartha mohanty, philip a. wilsey.
--smohanty@thor.ece.uc.edu, phil.wilsey@uc.edu


package petri is


--this package defines a set of data types and operations designed to
--support the description of marked petri nets in vhdl.  this package
--is designed to work in conjunction with two design entities: place
--and transition.  these two design entities are instantiated to build
--a petri net model.  


--the value of max_places_and_transitions can be as large as
--necessary.  however, because of the timing model used (minor/major
--cycles), the maximum number of transitions is limited to 1,000,000.

constant max_places_and_transitions : integer := 1000;


--debugging control values passed into places or transitions through
--the generics.  
--  none: no debuging output
--  handshaking: report handshaking values
--  firing: report firing actions
--  trace_tokens: report token migration
--  everything: report all actions

type debug is (none, handshaking, firing, trace_tokens, everything);

--used to identify component to print_debug

type debug_type is (id_a_place, id_a_transition);

--various types of Petri Nets, live, general, etc. Add to this
--as more implementations are made.

type p_type is (live, general);

--basic definition of token data structure and token handshaking
--values. 

type token_handshake is (not_driving, identify_place, identify_transition,
                             send, present, sending, firing);

type token is
record
  handshake : token_handshake;
  number : integer;
end record;


--set of overloaded functions to convert information to strings for
--printing purposes.

function stringof (handshake : token_handshake) return string;
function stringof (in_number : integer) return string;
function stringof (wk_time : time) return string;


--procedure to write debug line to stdout.

procedure print_debug (petri_component : debug_type;
                       debug_control : debug;
                       component_name : string;		       
		       component_id : integer;
		       message : string);


--type declarations for resolution functions.

type token_array is array (positive range <>) of token;
type integer_array is array (positive range <>) of integer;
type bool_array is array (positive range <>) of boolean;


--this procedure is used to resolve multiple drivers to the ports of a
--place and transition.  

function resolve_arc (input_tokens : token_array) return token;


--type declarations for input and output to transitions and places.

subtype arc is resolve_arc token;
type arc_array is array (positive range <>) of arc;


--type declarations to trace connected input/output places or
--transitions. 

subtype arc_list_element is integer;
type arc_list is array (positive range <>) of arc_list_element;

end petri;
