
--Copyright (c) 1993 Ohio Board of Regents and the University of
--Cincinnati.  All Rights Reserved.

--author: sidhartha mohanty, philip a. wilsey.
--smohanty@thor.ece.uc.edu, phil.wilsey@uc.edu


use std.textio.all;
package body petri is


--set of overloaded functions to convert information to strings for
--printing purposes.

--convert token handshaking enumeration types.

function stringof (handshake : token_handshake) return string is

type token_handshake_array is 
    array (token_handshake) of string (1 to 19);
constant token_handshake_strings : token_handshake_array := (
    "not_driving        ", 
    "present            ", 
    "send               ", 
    "sending            ", 
    "firing             ", 
    "identify_place     ",
    "identify_transition"
);

variable i : integer;
variable out_string : string (1 to 19);

begin

    i := 1;
    out_string := token_handshake_strings(handshake);

    while i < 19 and out_string(i+1) /= ' ' loop
    	i := i + 1;
    end loop;

    return out_string(1 to i);

end stringof;

--convert integers.

function stringof (in_number : integer) return string is

variable line, reverse_line : string (1 to 10) := "          ";
variable  i, j, wk_number : integer;
type char_array is array (integer range 0 to 9) of character;
variable digit_to_letter : char_array := ('0', '1', '2', '3', '4',
                                          '5', '6', '7', '8', '9');

begin

    if in_number = 0 then 
    	return "0";
    end if;
    i := 1;
    wk_number := in_number;
    while wk_number > 0 loop
    	reverse_line(i) := digit_to_letter(wk_number mod 10);
	wk_number := wk_number / 10;
	i := i + 1;
    end loop;

    j := 1;
    while j < i loop
    	line(j) := reverse_line(i - j);
	j := j + 1;
    end loop;
    
    return line(1 to j-1);

end stringof;


function stringof (wk_time : time) return string is

begin

    return stringof(wk_time / 0.000001 ns) & " fs";  --convert to integer and
					      --then use integer fn

end stringof;


--write string to stdout with newline.

procedure print_formatted_message (message : string) is

    variable message_line : line;

begin

    write(message_line, message);
    writeline(std.textio.output, message_line);

end print_formatted_message;


--write debug message to stdout with newline.

procedure print_debug (petri_component : debug_type;
                       debug_control : debug;
                       component_name : string;
		       component_id : integer;
		       message : string) is

    type debug_types is array (debug_type) of string (1 to 7);
    constant debug_type_name : debug_types := ("Place: ", "Trans: ");

    function everything_info (debug_control : debug) return string is
    	begin
	    if debug_control = everything then
	    	return "At (" & stringof(now) & ").  ";
	    else
	    	return "";
	    end if;
    end everything_info;

    function print_name (name : string) return string is
    	begin
	    if name /= "" then
	    	return name & ", ";
	    else
	    	return "";
	    end if;
    end print_name;
    
begin

    print_formatted_message(debug_type_name(petri_component) &
                                "(" & print_name(component_name) &
				stringof(component_id) & ").  " &
				everything_info(debug_control) &
				message);

end print_debug;


--this procedure is used to resolve multiple drivers the ports of a
--place or transition.
--when invoked, this function insures the consistency of handshaking
--values in token drivers.

function resolve_arc (input_tokens : token_array) return token is 
	variable token_hand	: token_handshake;
	variable out_token 	: token;

begin

--    if input_tokens'length = 0 then
--       return (not_driving, 0);
--    end if ;    

--    assert input_tokens'length <= 2
--      report "Each arc should connect only one place and transition."
--      severity failure;

--    if input_tokens'length = 1 then
--    	return input_tokens(1);
--    elsif input_tokens(1).handshake = not_driving then
--    	return input_tokens(2);
--    else 
--    	return input_tokens(1);
--    end if;

	token_hand	:= not_driving;
	out_token 	:= (not_driving, 0);

    for I in input_tokens'range loop
	if input_tokens(i).handshake > token_hand then
            out_token := input_tokens(i);          
	    return out_token;
        end if;
    end loop;

    return out_token;

end resolve_arc;

end petri;
  
